using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Unibase.Platform.Data;
using Unibase.Platform.Managers;

namespace Bizgaze.CRM.Visitors.Managers
{
    public partial interface IVisitorManager : IManager
    {
       Task<SelectResponse<Unibase.Platform.Templates.Infos.PrintTemplate>> GetVisitorshtmlAsync(SelectRequest srequest, long visitorid);
        Task<SelectResponse<Bizgaze.CRM.Visitors.Infos.Visitor>> GetpreVisitorsAsync(SelectRequest srequest, string code);
        Task<ActionResponse<Bizgaze.CRM.Visitors.Infos.Visitor>> VisitorAgreementAsync(ActionRequest arequest, long VisitorId);
        Task<ActionResponse<Bizgaze.CRM.Visitors.Infos.Visitor>> checkMobileNumberAsync(ActionRequest arequest, string mobileNumber, string emailId);
        Task<SelectResponse<Bizgaze.CRM.Visitors.Infos.Visitor>> GetVisitorsAsync(SelectRequest srequest, long VisitorId);
        Task<SelectResponse<Bizgaze.CRM.Visitors.Infos.Visitor>> GetInstallappAsync(SelectRequest srequest, long VisitorId);
        Task<SelectResponse<Bizgaze.CRM.Visitors.Infos.Visitor>> VisitorsbyMobileNumberAsync(SelectRequest srequest, string MobileNumber);
        //Task<ActionResponse<Infos.Visitor>> SaveVisitorAsync(ActionRequest request,long VisitorId);
    }
}
